@@ -22,7 +22,8 @@ LOGGING = {
 
 ANSIBLE_API_HOSTNAME = "https://" + socket.getfqdn()
 ANSIBLE_CONTENT_HOSTNAME = settings.CONTENT_ORIGIN + "/pulp/content"
+ANSIBLE_SIGNATURE_REQUIRE_VERIFICATION = True
 ANSIBLE_SIGNING_TASK_LIMITER = 10
-ANSIBLE_CERTS_DIR = "/etc/pulp/certs/"
 ANSIBLE_DEFAULT_DISTRIBUTION_PATH = None
 ANSIBLE_URL_NAMESPACE = ""
+ANSIBLE_COLLECT_DOWNLOAD_LOG = False
