@@ -139,13 +139,13 @@ def promote_content(repos_per_task, num_repos_to_update):
         locks.append(repo)
         if len(repos_to_dispatch) == repos_per_task:
             task_args = (random_collection_version.pk, repos_to_dispatch)
-            dispatch(add_content_to_repositories, locks, args=task_args)
+            dispatch(add_content_to_repositories, exclusive_resources=locks, args=task_args)
             repos_to_dispatch = []
             locks = []
 
     if repos_to_dispatch:
         task_args = (random_collection_version.pk, repos_to_dispatch)
-        dispatch(add_content_to_repositories, locks, args=task_args)
+        dispatch(add_content_to_repositories, exclusive_resources=locks, args=task_args)
 
 
 def add_content_to_repositories(collection_version_pk, repositories_pks):
